/**
  ##############################################################################
	@file    modbus_config.h
	@brief   Все define для адресов, стандартных значений и типов регистров

  ##############################################################################
*/

#ifndef MODBUS_CONFIG_H_
#define MODBUS_CONFIG_H_

#include "platform.h"
#include "modbus_regs_file.h" 

#define MBADDR_BAUD_RATE 0x006E 
#define MBTYPE_BAUD_RATE MBR_holding
#define MBDDATA_BAUD_RATE 96u

#define MBADDR_PARITY_BIT 0x006F
#define MBTYPE_PARITY_BIT MBR_holding
#define MBDDATA_PARITY_BIT 0u

#define MBADDR_STOP_BIT 0x0070
#define MBTYPE_STOP_BIT MBR_holding
#define MBDDATA_STOP_BIT 2u

#define MBADDR_SLAVE_ADDR 0x0080
#define MBTYPE_SLAVE_ADDR MBR_holding
#define MBDDATA_SLAVE_ADDR 0x4A


#define MBADDR_TIME_SEC0 0x0068
#define MBADDR_TIME_SEC1 0x0069
#define MBTYPE_TIME_SEC MBR_input


#define MBADDR_STATE_DOUT 0x0012
#define MBDDATA_STATE_DOUT 0x0000 //modbus default data
#define MBTYPE_STATE_DOUT MBR_coils


#define MBADDR_STATUS_LED 0x0082
#define MBDDATA_STATUS_LED 0x0000 //modbus default data
#define MBTYPE_STATUS_LED MBR_holding


#define MBADDR_TEMPERATURE  0x0
#define MBDDATA_TEMPERATURE 0x7FFF
#define MBTYPE_TEMPERATURE MBR_input

#define MBADDR_TEMPERATURE_COMPENSATION 0x00F5
#define MBDDATA_TEMPERATURE_COMPENSATION 0x0
#define MBTYPE_TEMPERATURE_COMPENSATION MBR_holding

#define MBADDR_HUMIDITY  0x1
#define MBDDATA_HUMIDITY 0x7FFF
#define MBTYPE_HUMIDITY MBR_input


#define MBADDR_RESET  0x0078
#define MBDDATA_RESET 0x0000
#define MBTYPE_RESET MBR_holding

#define MBADDR_MODEL 0x00C8
#define MBDDATA_MODEL 0x5465,0x6D70,0x436F,0x6E74,0x726F,0x6C00 //Модель устройства (строка asci "TempControl") адресса 200-205
#define MBTYPE_MODEL MBR_input


//для записи в еепром если не нашли существующих настроек
//чтоб добавить регистр в файл измените DEFAULT_FILE_NUMS_ADDR и DEFAULT_FILE_NUMS_ADDR возможно нужно изменить EEPROM_BUF_SIZE
#define FIRMWARE_VERSION 0x0020 //это пока что для формальности. надо родумать и заменить/перенести
#define EEPROM_S_DATA_FLAG 0xA5 
#define EEPROM_E_DATA_FLAG 0x6E
#define LENGTH_INIT_DATA 4u //EEPROM_S_DATA_FLAG*2 + DEFAULT_FILE_NUMS_ADDR + FIRMWARE_VERSION
#define LENGTH_END_DATA 2u //EEPROM_E_DATA_FLAG*2
#define ONE_REG_SIZE 5u //2 байта адреса + 2 байт типа + 2 байта данных

#define U16_TO_U8_OLDER_BYTE(b) ((uint8_t)((uint16_t)(b)>>8u))
#define OB(b) U16_TO_U8_OLDER_BYTE(b) 
#define OJ(b) (uint8_t)(b) 

#define DEFAULT_FILE_START_ADDR0 (uint8_t)0x0 
#define DEFAULT_FILE_START_ADDR1 (uint8_t)0x0 
#define DEFAULT_FILE_NUMS_ADDR 7u //кол-во регистров в файле

#define DEFAULT_FILE 	EEPROM_S_DATA_FLAG,EEPROM_S_DATA_FLAG,FIRMWARE_VERSION,DEFAULT_FILE_NUMS_ADDR, \
						OB(MBADDR_BAUD_RATE),OJ(MBADDR_BAUD_RATE),MBTYPE_BAUD_RATE,OB(MBDDATA_BAUD_RATE),OJ(MBDDATA_BAUD_RATE), \
						OB(MBADDR_PARITY_BIT),OJ(MBADDR_PARITY_BIT),MBTYPE_PARITY_BIT,OB(MBDDATA_PARITY_BIT),OJ(MBDDATA_PARITY_BIT), \
						OB(MBADDR_STOP_BIT),OJ(MBADDR_STOP_BIT),MBTYPE_STOP_BIT,OB(MBDDATA_STOP_BIT),OJ(MBDDATA_STOP_BIT), \
						OB(MBADDR_SLAVE_ADDR),OJ(MBADDR_SLAVE_ADDR),MBTYPE_SLAVE_ADDR,OB(MBDDATA_SLAVE_ADDR),OJ(MBDDATA_SLAVE_ADDR), \
						OB(MBADDR_TEMPERATURE_COMPENSATION),OJ(MBADDR_TEMPERATURE_COMPENSATION),MBTYPE_TEMPERATURE_COMPENSATION,OB(MBDDATA_TEMPERATURE_COMPENSATION),OJ(MBDDATA_TEMPERATURE_COMPENSATION), \
						OB(MBADDR_STATE_DOUT),OJ(MBADDR_STATE_DOUT),MBTYPE_STATE_DOUT,OB(MBDDATA_STATE_DOUT),OJ(MBDDATA_STATE_DOUT), \
						OB(MBADDR_STATUS_LED),OJ(MBADDR_STATUS_LED),MBTYPE_STATUS_LED,OB(MBDDATA_STATUS_LED),OJ(MBDDATA_STATUS_LED), \
						EEPROM_E_DATA_FLAG,EEPROM_E_DATA_FLAG

#endif