/**
  ******************************************************************************
  * @file    crc.c
  * @brief   CRC generation for modbus
  ******************************************************************************
*/

#include "platform.h"
#include "crc.h"

#define MB_POLY 0x8005 //reversed A001
#define SHT4X_POLY 0x13 //reversed 0x31

static void CRC_Init(void);


/**
    @brief init
 */
static void CRC_Init(void)
{
    RCC->AHBENR |= RCC_AHBENR_CRCEN;
    CRC->CR &= ~(CRC_CR_POLYSIZE | CRC_CR_REV_IN);
    CRC->CR |= (CRC_CR_POLYSIZE_0 | CRC_CR_REV_OUT | CRC_CR_REV_IN_0);
    CRC->POL = MB_POLY;
    CRC->CR |= CRC_CR_RESET;
}

/**
   @brief считавем CRC  
   @param buffer указатель на массив с данными (предполагается modbus frame) 
   @param len длина массива buffer[len-1 должен указыть на последний байт CRC]
   @param crc_h_byte указать, по которому положим h байт посчитанной CRC
   @param crc_l_byte указать, по которому положим l байт посчитанной CRC
   @note l byte это последний по счёту байт
 */
void CRC_get(uint8_t * buffer, uint32_t len, uint8_t* crc_h_byte, uint8_t* crc_l_byte)
{
	CRC_Init();
    for(uint32_t i = 0; i < (len-CRC_LEN); i++)
    {
        *((__IO uint8_t *)&CRC->DR) = buffer[i];
    }
    *crc_h_byte=(uint8_t)(CRC->DR);
    *crc_l_byte=(uint8_t)(CRC->DR>>8);
    return;
}

/**
   @brief вычитает CRC из массива и сравнивает их c СRC внутри массива (последнии 2 байта)
   @param buffer указатель на массив с данными (предполагается modbus frame) 
   @param len длина массива buffer[len-1 должен указыть на последний байт CRC]
   @return 1 если CRC совпадают
 */
bool CRC_check(uint8_t * buffer, uint32_t len)
{
    uint8_t crc_h_buf,crc_l_buf;
    CRC_get(buffer, len,&crc_h_buf,&crc_l_buf);
    return (crc_h_buf==buffer[len-CRC_LEN])&&(crc_l_buf==buffer[len-1u]); 
}
