#include "platform.h"
#include "led.h"
#include "modbus_regs_file.h"
#include "modbus_config.h"

static uint16_t led_state_flag = MBDDATA_STATUS_LED;

/**
   @brief инициализируем светодиод
 */
void led_config(void)
{
    RCC   -> IOPENR  |= RCC_IOPENR_GPIOAEN ;

    GPIOA -> MODER   &= ~GPIO_MODER_MODE7_1;
    GPIOA -> MODER   |= GPIO_MODER_MODE7_0; //General purpose output mode

    GPIOA -> OTYPER  |= GPIO_OTYPER_OT1; //Output open-drain

    GPIOA -> OSPEEDR &= ~GPIO_OSPEEDR_OSPEED7_1;
    GPIOA -> OSPEEDR &= ~GPIO_OSPEEDR_OSPEED7_0; //Very low speed

    GPIOA -> PUPDR   &= ~ GPIO_PUPDR_PUPD7_1;  //Pull-up
    GPIOA -> PUPDR   |= GPIO_PUPDR_PUPD7_0;

    led_set(led_state_flag); 
    return;
}

/**
   @brief переключить светодиод
   @note меняет начение модбас регистра
 */
void led_set(bool state)
{
    if(state!=LED_OFF){
        GPIOA -> ODR &= ~GPIO_ODR_OD7;
        led_state_flag = LED_ON;
    }else{
        GPIOA -> ODR |=GPIO_ODR_OD7;
        led_state_flag = LED_OFF;
    }
    return;
}

/**
   @brief получить текущее состояние светодиода  
 */
bool led_get(void)
{
    return !(led_state_flag!=LED_OFF);
}

/**
   @brief синхронизируем значение регистра модас со светом  
 */
void led_periodic_work(void)
{
    led_set(TO_BOOL(led_state_flag));
}

/**
   @brief передаём указатели в модбас
 */
uint8_t MBRF_led_init(void)
{
    return  MBRF_create_reg(MBADDR_STATUS_LED, &led_state_flag, MBTYPE_STATUS_LED); //0x0012
}