/**
  ******************************************************************************
  * @file    rcc.c
  * @brief   Clock setup
  ******************************************************************************
  */

#include "rcc.h"
#include "platform.h"

static uint32_t sysClockFreq = 0 ; //тут храним текущую частоту 

/**
  	@brief Set SYSCLK 64mhz from PLL <- HSI16
  */
void sysclk_64mhz_from_hsi16_setup(void)
{
	// RCC -> CR |= RCC_CR_HSIDIV_0; //поставить на 8мгц делитель HSI16

	RCC -> CR &= ~RCC_CR_PLLON; //Disable the PLL
	while((RCC -> CR&RCC_CR_PLLRDY)!=0) //Wait until PLLRDY is cleared. The PLL is now fully stopped
	{NOP();}
	RCC -> PLLCFGR =0U; //reset config PLL
	RCC -> PLLCFGR |=RCC_PLLCFGR_PLLR_0;  //PLL VCO division factor R for PLLRCLK clock output = 2
	RCC -> PLLCFGR |=RCC_PLLCFGR_PLLN_3; // PLL frequency multiplication factor N = 8
	RCC -> PLLCFGR |=RCC_PLLCFGR_PLLSRC_1; //PLL input clock source = HSI16
	RCC -> CR |=RCC_CR_PLLON; //Enable the PLL
	while((RCC -> CR&RCC_CR_PLLRDY)!=0) //Wait PLL start
	{NOP();}
    FLASH->ACR |= FLASH_ACR_LATENCY_2; //Flash memory access latency = Two wait states
	RCC -> PLLCFGR |=RCC_PLLCFGR_PLLREN; //PLLRCLK clock output enable
	RCC -> CFGR |= RCC_CFGR_SW_1; //System clock switch = PLLRCLK

	sysClockFreq = 64000000; //устанавливаем частоту в 64мгц 

	return;
}

/**
  	@brief Enable LSI
  */
void sysclk_LSI_setup(void)
{
	RCC -> CSR |= RCC_CSR_LSION; //LSI oscillator enable
	while( (RCC -> CSR & RCC_CSR_LSIRDY) == 0x0)
	{
		NOP();
	}
	return;
}


/**
  	@brief Setup microcontroller clock output in PA8
  	prescaler = 2 
  	bare-metal code because used at debug only
  	@param[selection] Microcontroller clock output clock selector
  */
void clock_output_pA8_enble(MSO_Sel_TypeDef selection)
{
	RCC   -> IOPENR  |= RCC_IOPENR_GPIOAEN ; //enable rcc A port

	GPIOA -> MODER   |= GPIO_MODER_MODE8_1;  // Alternate function mode (10)
	GPIOA -> MODER   &= ~GPIO_MODER_MODE8_0; 	
	GPIOA -> OSPEEDR |= GPIO_OSPEEDR_OSPEED8_1;
	GPIOA -> OSPEEDR |= GPIO_OSPEEDR_OSPEED8_0; //Very high speed


	RCC  -> CFGR &= ~RCC_CFGR_MCOPRE_Msk; //Microcontroller clock output prescaler = 2
	RCC  -> CFGR |=RCC_CFGR_MCOPRE_0;

	RCC -> CFGR &= ~RCC_CFGR_MCOSEL_Msk; //Microcontroller clock output selector 
	RCC -> CFGR |=((selection << RCC_CFGR_MCOSEL_Pos)&RCC_CFGR_MCOSEL_Msk); //сдвигаем и по маске пишем

	return;
}

/**
  	@brief return sysClockFreq
  	@return sysClockFreq
  */
uint32_t RCC_GetSysClockFreq(void)
{
	return sysClockFreq;
}