   #include "uart_debug.h"
#include "platform.h"
#include "xprintf.h"

static void uart_putc(uint8_t d);
static uint8_t uart_getc(void);

static void printf_config(void)
{
    xdev_out(uart_putc);
    xdev_in(uart_getc);
    return;
}

static uint8_t uart_getc(void)
{
    while ((USART2->ISR & USART_ISR_RXNE_RXFNE) == 0) { NOP();}
    return USART2->RDR; 
}

static void uart_putc(uint8_t ch)
{
    while ((USART2->ISR & USART_ISR_TC) == 0) { NOP();}
    USART2->TDR = ch;
}

/**
    @brief инициализируем в uart в блокирующем режиме в передаём уазатели в xprintf
 */
void uart_debug_init(void)
{
    RCC->APBENR1 |= RCC_APBENR1_USART2EN; // Включаем тактирование UART2
    RCC->IOPENR  |= RCC_IOPENR_GPIOAEN;   // Включаем тактирование GPIO порта A

    SYSCFG->CFGR2 |= SYSCFG_CFGR2_PA3_CDEN; //Clamping Diode Enable
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_PA13_CDEN; //Clamping Diode Enable на SWDIO //по хорошему это нужно делать не тут

    // Настройка PA2 как TX и PA3 как RX
    GPIOA->MODER &= ~(GPIO_MODER_MODE2_0 | GPIO_MODER_MODE3_0); // Режим альтернативной функции
    GPIOA->MODER |= (GPIO_MODER_MODE2_1 | GPIO_MODER_MODE3_1);  // Установить режим альтернативной функции

    GPIOA->AFR[0] |= GPIO_AFRL_AFSEL2_0 | GPIO_AFRL_AFSEL3_0; // Установить альтернативную функцию AF1 для PA2 и PA3

    // Настройка скорости GPIO
    GPIOA->OSPEEDR &= ~(GPIO_OSPEEDR_OSPEED2 | GPIO_OSPEEDR_OSPEED3); // Настройка на очень низкую скорость

    USART2->CR1 = 0; // Сбрасываем регистр CR1
    USART2->BRR = 556; // Установка BRR для скорости 115200 при 64 МГц
    USART2->CR1 |= USART_CR1_TE | USART_CR1_RE; // Разрешаем передатчик и приемник
    USART2->CR1 |= USART_CR1_UE; // Включаем USART2
    USART2->CR2 = 0; // Сбрасываем регистр CR2
    USART2->CR3 = 0; // Сбрасываем регистр CR3

    printf_config();
}